import {
  ConflictException,
  Injectable,
  NotFoundException,
  UnauthorizedException,
} from '@nestjs/common';
import { InjectRepository } from '@nestjs/typeorm';
import { Repository } from 'typeorm';
import { Operador } from './entity/operador.entity';
import { InstitucionService } from 'src/institucion/institucion.service';
import { TipoUsuarioService } from 'src/tipo-usuario/tipo-usuario.service';

@Injectable()
export class OperadorService {
  constructor(
    @InjectRepository(Operador) private repository: Repository<Operador>,
    private isnstitucionService: InstitucionService,
    private tipoUsuarioService: TipoUsuarioService,
  ) {}

  async create(id_institucion: number, operador: string, password: string) {
    const institucion = await this.isnstitucionService.findById(id_institucion);
    const tipoUsuario = await this.tipoUsuarioService.findById(4);

    return this.repository
      .findOne({ operador, institucion })
      .then((existeOperador) => {
        if (existeOperador)
          throw new ConflictException(
            'Ya existe un operador con ese nombre, intente de nuevo.',
          );
        /* Encriptar password */
        return this.repository.save(
          this.repository.create({
            institucion,
            operador,
            password,
            tipoUsuario,
          }),
        );
      })
      .then((_) => ({ message: 'Se creo correctamente al operador.' }));
  }

  async findAll(filtros: {
    pagina: string;
    id_institucion?: string;
    operador?: string;
  }) {
    const busqueda: any = {};
    const institucion = filtros.id_institucion
      ? await this.isnstitucionService.findById(Number(filtros.id_institucion))
      : null;

    if (filtros.operador) busqueda.operador = filtros.operador;
    if (filtros.id_institucion) busqueda.institucion = institucion;
    return this.repository.find(busqueda);
  }

  findById(id_operador: number) {
    return this.repository.findOne({ id_operador }).then((operador) => {
      if (!operador) throw new NotFoundException('No existe este operador');
      return operador;
    });
  }

  findByOperador(operador: string) {
    return this.repository.findOne({ operador }).then((operador) => {
      if (!operador) throw new NotFoundException('No existe este operador');
      return operador;
    });
  }

  login(operador: string, password: string) {
    return this.findByOperador(operador).then((operador) => {
      if (operador.password !== password)
        throw new UnauthorizedException('Contraseña incorrecta.');
      /* Crear JWT y regresarlo */
      return operador;
    });
  }

  async update(attrs: Partial<Operador>) {
    return this.findById(attrs.id_operador)
      .then((operador) => {
        /* encriptar password */
        Object.assign(operador, attrs);
        return this.repository.save(operador);
      })
      .then((_) => ({
        message: 'Se actualizo correctamente la información del operador.',
      }));
  }
}
