import {
  Column,
  Entity,
  JoinColumn,
  ManyToOne,
  PrimaryGeneratedColumn,
} from 'typeorm';
import { Equipo } from '../../equipo/entity/equipo.entity';
import { Operador } from '../../operador/entity/operador.entity';
import { Usuario } from '../../usuario/entity/usuario.entity';

@Entity()
export class Prestamo {
  @PrimaryGeneratedColumn()
  id_prestamo: number;

  @Column({ type: Boolean, nullable: false, default: true })
  activo: boolean;

  @Column({ type: Boolean, nullable: false, default: false })
  cancelado_operador: boolean;

  @Column({ type: Boolean, nullable: false, default: false })
  cancelado_usuario: boolean;

  @Column({ type: Date, nullable: false })
  fecha_entrega: Date;

  @Column({ type: Date, nullable: false })
  fecha_inicio: Date;

  @Column({ type: Date, nullable: false })
  hora_fin: Date;

  @Column({ type: Date, nullable: false })
  hora_inicio: Date;

  @Column({ type: Date, nullable: false })
  hora_max_recoger: Date;

  @ManyToOne(() => Equipo, (equipo) => equipo.prestamos)
  @JoinColumn({ name: 'id_equipo' })
  equipo: Equipo;

  @ManyToOne(
    () => Operador,
    (operadorEntrega) => operadorEntrega.prestamosOperadorEntrega,
  )
  @JoinColumn({ name: 'id_operador_entrega' })
  operadorEntrega: Operador;

  @ManyToOne(
    () => Operador,
    (operadorRegreso) => operadorRegreso.prestamosOperadorRegreso,
  )
  @JoinColumn({ name: 'id_operador_regreso' })
  operadorRegreso: Operador;

  @ManyToOne(() => Usuario, (usuario) => usuario.prestamos)
  @JoinColumn({ name: 'id_usuario' })
  usuario: Usuario;
}
