import {
  Injectable,
  NotFoundException,
  UnauthorizedException,
} from '@nestjs/common';
import { InjectRepository } from '@nestjs/typeorm';
import { Repository } from 'typeorm';
import { Usuario } from './entity/usuario.entity';
import { InstitucionService } from '../institucion/institucion.service';
import { TipoUsuarioService } from '../tipo-usuario/tipo-usuario.service';

@Injectable()
export class UsuarioService {
  constructor(
    @InjectRepository(Usuario) private repository: Repository<Usuario>,
    private institucionService: InstitucionService,
    private tipoUsuarioService: TipoUsuarioService,
  ) {}

  async findAll(filtros: {
    pagina: string;
    id_carrera?: string;
    id_institucion?: string;
    id_tipo_usuario?: string;
    usuario?: string;
  }) {
    const institucion = filtros.id_institucion
      ? this.institucionService.findById(Number(filtros.id_institucion))
      : null;
    const tipoUsuario = filtros.id_tipo_usuario
      ? this.tipoUsuarioService.findById(Number(filtros.id_tipo_usuario))
      : null;
    const updtae = {};
    /* Terminar cuando se tengan las fuinciones faltantes */
  }

  findById(id_usuario: number) {
    return this.repository.findOne({ id_usuario }).then((usuario) => {
      if (!usuario) throw new NotFoundException('No existe este usuario.');
      return usuario;
    });
  }

  findByUsuario(usuario: string) {
    return this.repository.findOne({ usuario }).then((usuario) => {
      if (!usuario) throw new NotFoundException('No existe este usuario.');
      return usuario;
    });
  }

  login(usuario: string, password: string) {
    return this.findByUsuario(usuario).then((usuario) => {
      if (password !== usuario.password)
        throw new UnauthorizedException('Contraseña incorrecta.');
      /* Crear JWT y regresarlo */
      return usuario;
    });
  }

  registrar(id_usuario: number, telefono: string) {
    return this.findById(id_usuario)
      .then((usuario) => {
        usuario.telefono = telefono;
        /* crear contraseña, encriptarla y mandarla por correo */
        usuario.password = 'hola';
        return this.repository.save(usuario);
      })
      .then((_) => ({ message: 'Se creo correctamente un usuario.' }));
  }

  async update(attrs: Partial<Usuario>) {
    return this.findById(attrs.id_usuario)
      .then((usuario) => {
        /* crear y encriptar password en caso de cambio de password */
        Object.assign(usuario, attrs);
        return this.repository.save(usuario);
      })
      .then((_) => ({
        message: 'Se actualizo correctamente la información del usuario.',
      }));
  }
}
